// current.cpp
// Draws the current diagrams for xfig

/* format:
#FIG 3.2
Landscape
Center
Inches
Letter  
100.00
Single
-2
1200 2
# The start of the specific objects
5 1 0 1 0 7 50 0 -1 0.000 0 0 0 0 8400.000 4800.000 8400 3600 9600 4800 7200 4800
#Draws an arc, last numbers mean: xcenter,ycenter,x1,y1,x2,y2,x3,y3
#4th value is its thickness
1 3 0 10 0 7 50 0 -1 0.000 1 0.0000 3600 2400 600 600 3600 2400 4200 2400
#Draws a circle, last numbers are xcenter,ycenter,xaxis,yaxis,x1,y1,x2,y2
2 1 0 1 0 7 50 0 -1 0.000 0 0 -1 1 0 2
	1 1 1.00 60.00 120.00
	 3600 3600 4800 3600
#Third line contains the (x1 y1 x2 y2) endpoints 
#Second line contains the arrow information.
#First line's 4th value is the thickness
*/
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#include <math.h>
#include <stdlib.h>
/*
#include <string.h>
#include "function.cpp"
*/




const double pi=3.1415;

double r[2000], v[2000];
int indmax;
int res;
double rmax,R;
double vmax;
double E;int m;
double N;
double xo,yo;

const int figc=1200;




double interpol(double x) {
  double rtick = (r[indmax]-r[0])/indmax;
  int i=int(x/rtick);
  return (-1./x)*(v[i]+ (x-i*rtick)*(v[i+1]-v[i]));
} 

class coord {
  private:

  public:
    double x,y;
    coord& operator = (const coord& P) {x=P.x;y=P.y; return *this;}
    coord() {}
    coord(const coord& P) {x=P.x;y=P.y;}
    coord(double x1,double y1) {x=x1;y=y1;}
    friend ostream& operator << (ostream&, coord);
};

coord polar(double r, double phi) {
   coord P(xo+r*cos(phi), yo+r*sin(phi));
   return P;
}

coord point(double x, double y) {
   coord P(x,y);
   return P;
}


ostream& operator << (ostream& os, coord P) {
   os << " " << int(figc*P.x) << " " << int(figc*P.y);
   return os;
}

void infromfile(char*);
void outtofile(char*, char*);
void classicalpath(double, ostream&);

int main(int argc, char *argv[]) {
  if (argc!=4) {
      cerr << "args: logfile resolution elements" << endl;
      exit(1);
  }
  res= atoi(argv[2]); N=atoi(argv[3]);

  ifstream logfile;
  logfile.open(argv[1], ios::in);
  if (!logfile) {
    cerr << "Error opening " << argv[1] << "!" << endl;
    exit(0);}  
  else  cout << endl << "Input values read from " << argv[1] << endl;
  int i=0;
  double s;
  char inputfilename[30]; char outputfilename[30]; char outputfilename0[30];
  while (logfile >> E >> m >> s >> inputfilename 
	 >> outputfilename0 >> outputfilename >> rmax) {
    i++; 
    R=sqrt(2*s);
    xo=2*rmax; yo=2*rmax;
    infromfile(inputfilename);
    outtofile(outputfilename0,outputfilename);
  }
/*
  if (argc!=9) {
      cerr << "args: E m s inputfile outputfile rmax resolution elements" << endl;
      return 0;}
  E=atof(argv[1]);m=atoi(argv[2]);

  rmax=atof(argv[6]);res=atoi(argv[7]);
  N=atoi(argv[8]);*/
}



void classicalpath(double phi0, ostream& os){
  double sigma=sqrt(E);
  double gamma=sqrt(E-2*m+R*R);
  double phi1=2*acos((R*R+gamma*gamma-sigma*sigma)/(2*gamma*R));
  double phi2=2*acos((sigma*sigma+R*R-gamma*gamma)/(2*sigma*R));
  //cerr << phi1/(2*pi) << " " << phi2/(2*pi) << endl;
  //cerr << sigma/R << " " << gamma/R << endl;
  cerr << "rho="<< sigma << "   c=" << gamma << "  R= " << R << "|  "<< phi1/pi << " " << phi2/pi<< endl;
  int thickness= 4;
  double phi=phi0;
  int i;
  os << "1 3 0 5 0 7 50 0 -1 0.000 1 0.0000" << coord(xo,yo)
     << coord(R,R) << coord(xo,yo) << coord(xo,yo-R) << endl;
  for (i=1;i<=N;i++) {
    os << "5 1 0 " << thickness << " 0 7 50 0 -1 0.000 0 1 1 0"  
       << polar(gamma,phi-phi1/2.) << polar(R,phi) 
       << polar(gamma+sigma,phi-phi1/2.) << polar(R,phi-phi1) << endl
       << "        1 0 4.00 90.00 180.00" << endl;
    os << "2 1 0 " << thickness << " 0 7 50 0 -1 0.000 0 0 -1 1 0 2" << endl
         << "        1 0 4.00 90.00 180.00" << endl
     << "        " << polar(R,phi-phi1) << polar(R,phi-phi1-phi2) << endl;
    phi-=phi1+phi2;
  }
}


void infromfile(char *inputfile) {
  ifstream infile;
  int i;
  infile.open(inputfile, ios::in);
  if (!infile) {
    cerr << "Error opening " << inputfile << "!" << endl;
    exit(0);}  
  else  cout << endl << "Input values read from " << inputfile << endl;

  i=0;
  while (infile >> r[i] >> v[i]) {
    i++;
  }
  indmax=i-1;
  infile.close();
  cout << indmax << "  ";
  vmax=0; 
  for (i=1;i<=indmax;i++) 
      if ((vmax<fabs(v[i]/r[i]))&&(r[i]>0.5*R))
	  vmax=fabs(v[i]/r[i]);
  cout << vmax << endl;
}

void outtofile(char* outfilename0, char* outfilename){
    ofstream outfile; ofstream outfile0;
  outfile.open(outfilename, ios::out);outfile0.open(outfilename0, ios::out);
  if (!outfile) {
    cerr << "Error opening " << outfilename << "!" << endl;
    exit(0);}  
  if (!outfile0) {
    cerr << "Error opening " << outfilename0 << "!" << endl;
    exit(0);}  
  outfile << "#FIG 3.2" << endl
	    << "Landscape" << endl 
	    << "Center" << endl
	    << "Inches" << endl
	    << "Letter" << endl
	    << "100.00" << endl
	    << "Single" << endl
	    << "-2" << endl
	    << "1200 2" << endl;
  outfile0 << "#FIG 3.2" << endl
	    << "Landscape" << endl 
	    << "Center" << endl
	    << "Inches" << endl
	    << "Letter" << endl
	    << "100.00" << endl
	    << "Single" << endl
	    << "-2" << endl
	    << "1200 2" << endl;
  int i,j;
  double xn,yn;
  double rho,phi;double L = rmax/res/vmax;double curr;
  // draws the anticircle contour
  classicalpath(0,outfile);
  classicalpath(0,outfile0);
  outfile0.close();
  cout << "Output written to " << outfilename0 << endl;
  for (i=-res;i<=res;i++) {
    for (j=-res;j<=res;j++) {
      if (i*i+j*j>=1) {
      xn = i*rmax/res; yn = j*rmax/res;
      rho = sqrt(xn*xn+yn*yn);
      phi = (xn>=0) ? atan(yn/xn) : atan(yn/xn)+pi;
      curr = interpol(rho);
      outfile << "2 1 0 1 0 7 50 0 -1 0.000 0 0 -1 1 0 2" << endl
	      << "1 1 " << fabs(curr)*L << " 60.00 120.00" << endl
              << "       " << coord(xn+xo,yn+yo) 
	      << coord(xn+xo-L*curr*sin(phi),yn+yo+L*curr*cos(phi)) << endl;
      }
      }
      }
  outfile.close();
  cout << "Output written to " << outfilename << endl;
}
